{

	// pt_CropLayerSets v1.2
	// Copyright (c) 2004-2008 Paul Tuersley. All rights reserved.
	// email: paul.tuersley@btinternet.com

	// Photoshop files can contain layer sets, groups of layers that are imported into
	// AE as precomps. These precomps are always the same size as the main comp.
	
	// This script crops these precomps to fit their contents, while retaining their
	// correct position in the main comp. Ideally the script should be run on a freshly
	// imported PSD comp before any changes are made.

	// Created based on a request from Mark Coleran.
	
	// v1.1 added support for nested groups / compositions in CS3. Ignores adjustment layers when calculating crop.
	// v1.2 fixed error caused by psd layers that were outside the canvas. These layers are now ignored in the crop calculation.
	


	function resizePrecomp(currentLayer) {

		var currentPrecomp = currentLayer.source
		if (currentPrecomp.numLayers == 0 ) { return; }

		var minX, maxX, minY, maxY;
		var thisLayer, layerPosition;
		var layerMinX, layerMaxX, layerMinY, layerMaxY;

		// find the first suitable layer to work out comp crop
		for (var j = 1; j <= currentPrecomp.numLayers; j++) {
			thisLayer = currentPrecomp.layer(j);

			// if this layer is a comp, recursive loop into this function to resize
			if (thisLayer.source instanceof CompItem) {
				resizePrecomp(thisLayer);
			}
			// ignore the layer if it's an adjustment layer
			if (thisLayer.adjustmentLayer == true) { continue; }
			
			// calculate the layer bounds
			layerPosition = thisLayer.property("Position").value;
			minX = layerPosition[0] - (thisLayer.source.width / 2);
			maxX = layerPosition[0] + (thisLayer.source.width / 2);
			minY = layerPosition[1] - (thisLayer.source.height / 2);
			maxY = layerPosition[1] + (thisLayer.source.height / 2);

			// ignore the layer if it's outside the comp boundary
			if (minX > currentPrecomp.width || maxX < 0 || minY > currentPrecomp.height || maxY < 0){
				continue;
			}

			// we've obviously found a suitable first layer, so start comparing to the other layers
			j++;
			while ( j <= currentPrecomp.numLayers) {

				thisLayer = currentPrecomp.layer(j);

				// recursive loop if this layer is a comp
				if (thisLayer.source instanceof CompItem) {
					resizePrecomp(thisLayer);
				}
				// ignore if it's an adjustment layer
				if (thisLayer.adjustmentLayer == true) {
					j++;
					continue;
				}
			
				layerPosition = thisLayer.property("Position").value;
				layerMinX = layerPosition[0] - (thisLayer.source.width / 2);
				layerMaxX = layerPosition[0] + (thisLayer.source.width / 2);
				layerMinY = layerPosition[1] - (thisLayer.source.height / 2);
				layerMaxY = layerPosition[1] + (thisLayer.source.height / 2);
				
				// ignore the layer if it's outside the comp boundary
				if (layerMinX > currentPrecomp.width || layerMaxX < 0 || layerMinY > currentPrecomp.height || layerMaxY < 0){
					j++
					continue;
				}

				// update the crop values if necessary, using the current layer's bounds
				if (layerMinX < minX) { minX = layerMinX; }
				if (layerMaxX > maxX) { maxX = layerMaxX; }
				if (layerMinY < minY) { minY = layerMinY; }
				if (layerMaxY > maxY) { maxY = layerMaxY; }

				j++;
			}

			// all layer's are compared
			// clamp the 'all layers extent' to the current comp size
			if ( minX < 0 ) { minX = 0; }
			if ( maxX > currentPrecomp.width ) { maxX = currentPrecomp.width; }
			if ( minY < 0 ) { minY = 0; }
			if ( maxY > currentPrecomp.height ) { maxY = currentPrecomp.height; }

			// find the center of the 'all layers extent'
			extentCenter[0] = ((maxX - minX) / 2 ) + minX;
			extentCenter[1] = ((maxY - minY) / 2 ) + minY;

			// find the relative offset to be applied to the layer in the main comp
			relativeOffset[0] = (currentPrecomp.width / 2) - extentCenter[0];
			relativeOffset[1] = (currentPrecomp.height / 2) - extentCenter[1];

			// set the relative offset position on the layer in the main comp
			layerPosition = currentLayer.property("Position").value;
			currentLayer.property("Position").setValue(layerPosition - relativeOffset);

			// resize the precomp to the 'all layers extent' size
			currentPrecomp.width = maxX - minX;
			currentPrecomp.height = maxY - minY;

			// find the relative offset to be applied to all layers in the precomp
			relativeOffset[0] = (currentPrecomp.width / 2) - extentCenter[0];
			relativeOffset[1] = (currentPrecomp.height / 2) - extentCenter[1];

			// set the relative offset position for each of the layers in the precomp
			for (var k = 1; k <= currentPrecomp.numLayers; k++) {
				thisLayer = currentPrecomp.layer(k);
				layerPosition = thisLayer.property("Position").value;
				thisLayer.property("Position").setValue(layerPosition + relativeOffset);
			}
		}
	}


	// main script
	app.beginUndoGroup("Crop Layer Sets");
	var activeItem = app.project.activeItem;
	//var currentLayer;
	relativeOffset = new Array();
	extentCenter = new Array();
	if (activeItem == null || !(activeItem instanceof CompItem) ) {
		alert("Import and select a PSD comp before running this script");
	} else {
		for (var i = 1; i <= activeItem.numLayers; i++) {
			currentLayer = activeItem.layer(i);
			if (currentLayer.source instanceof CompItem) {	
				resizePrecomp(currentLayer);		
			}
		}
	}
	app.endUndoGroup();
		
}